@@ -685,3 +685,389 @@ generate_cov_str <- function(p = 3,
     return(e)
   }
 }
+
+
+
+
+# Generate data for the robust multivariate functional control cha --------
+#' @noRd
+generate_cov_str_RoMFCC <- function(p = 3,
+                                    P = 100,
+                                    correlation = "decreasing",
+                                    k = 1) {
+  n_comp_x <- 100
+  x_seq <- seq(0, 1, l = P)
+
+  get_mat <- function(cov) {
+    P <- length(cov)
+    covmat <- matrix(0, nrow = P, ncol = P)
+    for (ii in 1:P) {
+      covmat[ii, ii:P] <- cov[1:(P - ii + 1)]
+      covmat[P - ii + 1, 1:(P - ii + 1)] <- rev(cov[1:(P - ii + 1)])
+    }
+    covmat
+  }
+  eig <- covmatList <- list()
+  cov1 <- besselJ(x_seq * 32, 0)
+  covmat1 <- get_mat(cov1)
+  eig1 <- eigen(covmat1)
+  w <- 1 / P
+  eig1$vectors <- eig1$vectors / sqrt(w)
+  eig1$values <- eig1$values * w
+  for (ii in 1:p) {
+    covmatList[[ii]] <- covmat1
+    eig[[ii]] <- eig1
+  }
+  eigList <- eig
+
+  eigenvalues <- rowMeans(sapply(1:p, function(ii)
+    eig[[ii]]$values))
+
+  corr_mat <- vector("list", p)
+  for (ii in 1:p) {
+    corr_mat[[ii]] <- vector("list", p)
+  }
+
+  for (ii in 1:p) {
+    for (jj in ii:p) {
+      if (jj == ii) {
+        corr_mat[[ii]][[jj]] <- covmatList[[ii]]
+      } else {
+        V <- eigList[[ii]]$vectors[, 1:n_comp_x]
+        lambdas <- eigenvalues[1:n_comp_x]
+
+        if (correlation == "decreasing") {
+          sum <- V %*% (t(V) * lambdas) / (1 + abs(ii - jj) / k)
+        }
+        if (correlation == "equal") {
+          sum <- V %*% (t(V) * lambdas) / (1 + 1 / k)
+        }
+        if (!(correlation %in% c("decreasing", "equal"))) {
+          stop("correlation must be either decreasing or equal")
+        }
+
+        corr_mat[[ii]][[jj]] <- sum
+        corr_mat[[jj]][[ii]] <- t(sum)
+      }
+    }
+    corr_mat[[ii]] <- do.call("cbind", corr_mat[[ii]])
+  }
+  corr_mat <- do.call("rbind", corr_mat)
+  e <- eigen(corr_mat)
+
+  list(e = e, P = P)
+}
+
+#' Simulate multivariate functional data with casewise and componentwise contamination
+#'
+#' Generate multivariate functional data under different contamination models
+#' (casewise, componentwise, or both) for use in simulation studies of the
+#' Robust Multivariate Functional Control Chart (RoMFCC) as described in
+#' Capezza, Centofanti, Lepore, and Palumbo (2024).
+#'
+#' The generated data mimic dynamic resistance curves (DRCs) in resistance
+#' spot welding processes and allow for controlled introduction of
+#' casewise and/or componentwise outliers, as in the Monte Carlo study
+#' presented in the RoMFCC paper.
+#'
+#' @param nobs Integer. Number of observations to simulate (default 1000).
+#' @param p Integer. Number of functional components (variables) (default 3).
+#' @param p_cellwise Numeric in \eqn{[0,1]}. Probability of cellwise contamination
+#'   (componentwise outliers) for each component (default 0).
+#' @param p_casewise Numeric in \eqn{[0,1]}. Probability of casewise contamination
+#'   (entire observation contaminated) (default 0).
+#' @param sd_e Numeric. Standard deviation of the additive measurement noise (default 0.005).
+#' @param sd Numeric. Standard deviation scaling of the functional part (default 0.002).
+#' @param T_exp Numeric in \eqn{(0,1)}. Expansion parameter for phase outliers (default 0.6).
+#' @param outlier Character. Type of contamination in Phase I sample:
+#'   \code{"no"} (no contamination),
+#'   \code{"outlier_M"} (mean shift),
+#'   \code{"outlier_E"} (exponential shift),
+#'   \code{"outlier_P"} (phase shift). Default \code{"no"}.
+#' @param M_outlier_case Numeric. Magnitude of casewise outlier contamination (default 0).
+#' @param M_outlier_cell Numeric. Magnitude of cellwise outlier contamination (default 0).
+#' @param OC Character. Out-of-control model in Phase II data:
+#'   \code{"no"} (in control),
+#'   \code{"OC_M"} (mean shift),
+#'   \code{"OC_E"} (exponential shift),
+#'   \code{"OC_P"} (phase shift). Default \code{"no"}.
+#' @param M_OC Numeric. Magnitude of out-of-control shift (default 0).
+#' @param P Integer. Number of grid points in each functional profile (default 100).
+#' @param max_n_cellwise Integer. Maximum number of components per observation
+#'   allowed to be cellwise contaminated (default \code{Inf}).
+#' @param correlation Character. Correlation structure among components:
+#'   typically \code{"decreasing"} (default).
+#' @param k Integer. Correlation parameter (default 1).
+#' @param which_OC Integer vector. Indices of components subject to out-of-control
+#'   shifts in Phase II (default 5).
+#'
+#' @details
+#' The function generates \code{nobs} realizations of a \code{p}-variate functional
+#' quality characteristic observed on an equally spaced grid of size \code{P}.
+#' The underlying process is simulated through a Karhunen–Loève expansion
+#' with eigenfunctions and eigenvalues derived from a specified correlation
+#' structure. Outliers can be introduced at cellwise level (single components),
+#' casewise level (entire observation), or both, using probability parameters
+#' \code{p_cellwise} and \code{p_casewise} and magnitudes
+#' \code{M_outlier_cell}, \code{M_outlier_case}.
+#' Out-of-control shifts in Phase II can be introduced via the \code{OC} argument.
+#'
+#' This setup mirrors the simulation design in Section 4 of
+#' Capezza et al. (2024) where RoMFCC was benchmarked against competing
+#' control charts under various contamination scenarios.
+#'
+#' @return A list with two elements:
+#' \describe{
+#'   \item{X_mat_list}{A list of length \code{p}, each element a matrix of
+#'     dimension \code{nobs × P} with the simulated functional observations.}
+#'   \item{ind_out}{A list of length \code{p}, each element containing the
+#'     indices of observations contaminated in that component.}
+#' }
+#'
+#' Capezza, C., Centofanti, F., Lepore, A., Palumbo, B. (2024)
+#' Robust Multivariate Functional Control Chart.
+#' \emph{Technometrics}, 66(4):531--547, <doi:10.1080/00401706.2024.2327346>.
+#'
+#' @examples
+#' \donttest{
+#' # Simulate uncontaminated data (Phase I)
+#' sim <- simulate_data_RoMFCC(nobs = 200, p = 3, outlier = "no", OC = "no")
+#' str(sim$X_mat_list)
+#'
+#' # Simulate with componentwise outliers in Phase I
+#' sim2 <- simulate_data_RoMFCC(nobs = 200, p = 3,
+#'                              p_cellwise = 0.05, M_outlier_cell = 0.03,
+#'                              outlier = "outlier_E")
+#'
+#' # Simulate Phase II with a mean shift in one component
+#' sim3 <- simulate_data_RoMFCC(nobs = 200, p = 3,
+#'                              OC = "OC_M", M_OC = 0.04, which_OC = 2)
+#' }
+#'
+#' @export
+simulate_data_RoMFCC <- function(nobs = 1000,
+                                 p = 3,
+                                 p_cellwise = 0,
+                                 p_casewise = 0,
+                                 sd_e = 0.005,
+                                 sd = 0.002,
+                                 T_exp = 0.6,
+                                 outlier = "no",
+                                 M_outlier_case = 0,
+                                 M_outlier_cell = 0,
+                                 OC = "no",
+                                 M_OC = 0,
+                                 P = 100,
+                                 max_n_cellwise = Inf,
+                                 correlation = "decreasing",
+                                 k = 1,
+                                 which_OC = 5) {
+  cov_str <- generate_cov_str_RoMFCC(p, P, correlation = correlation, k = k)
+  e <- cov_str$e
+  P <- cov_str$P
+  x_seq <- seq(0, 1, l = P)
+
+  w <- 1 / P
+  n_comp_x <- max(which((cumsum(e$values) / sum(e$values)) < 1))
+
+  meig <- list()
+  for (ii in 1:p) {
+    index <- 1:P + (ii - 1) * P
+    meig[[ii]] <- e$vectors[index, 1:n_comp_x] / sqrt(w)
+  }
+  meigenvalues <- e$values[1:n_comp_x] * w
+  meigenvalues[meigenvalues < 0] <- 0
+
+  csi_X <- stats::rnorm(
+    n = length(meigenvalues) * nobs,
+    mean = 0,
+    sd = sqrt(rep(meigenvalues, each = nobs))
+  )
+  csi_X <- matrix(csi_X, nrow = nobs)
+
+
+  fun_phase <- function(x_ini, M_g) {
+    x <- numeric(P)
+    T1 <- 0.0
+    T2 <- 0.6
+    a <- (T2 - M_g / 1.5 - T1) / (T2 - T1)
+    b <- T1 - a * T1
+    xx <- x_ini * a + b
+    a <- (T2 - M_g / 1.5 - 1) / (T2 - 1)
+    b <- 1 - a
+    xx2 <- x_ini * a + b
+    x[1:(T1 * P)] <- x_ini[1:(T1 * P)]
+    x[((T1 * P) + 1):(T2 * P)] <- xx[((T1 * P) + 1):(T2 * P)]
+    x[(T2 * P + 1):(P)] <- xx2[(T2 * P + 1):(P)]
+    x <-
+      (x - min(x)) / (max(x) - min(x)) * (0.15 - 0.0045) + 0.0045
+
+    x3 <- x_ini ^ (1 + M_g / 0.6)
+    x3 <-
+      (x3 - min(x3)) / (max(x3) - min(x3)) * (0.15 - 0.0045) + 0.0045
+    x <- x3
+
+    - (M_g / 5) * x_ini + 0.45 * (M_g / 5) + 0.2074 + 0.3117 * exp((-(371.4)) * x) + 0.5284 * (1 - exp((-(-0.8217)) * x)) - 423.3 * (1 + tanh(-26.15 * (x - (-0.1715))))
+  }
+  fun_m <- function(x_ini, M_g) {
+    x <- x_ini
+    x <- (x - min(x)) / (max(x) - min(x)) * (0.15 - 0.0045) + 0.0045
+    0.2074 + 0.3117 * exp((-(371.4)) * x) + 0.5284 * (1 - exp((-(-0.8217)) * x)) - 423.3 * (1 + tanh(-26.15 * (x - (-0.1715))))
+  }
+  if (OC == "OC_P") {
+    f1_m <- fun_m
+  } else {
+    f1_m <- fun_m
+  }
+
+  if (outlier == "no") {
+    cont_function <- function(nobs, p_g, M_g, T_exp)
+      0
+  } else if (outlier == "outlier_M") {
+    cont_function <- function(nobs, p_g, M_g, T_exp) {
+      rb <-
+        stats::rbinom(nobs, 1, p_g) * sample(c(-1, 1), nobs, replace = T)
+      M_g * matrix(rb , P, nobs, byrow = T)
+    }
+  } else if (outlier == "outlier_E") {
+    cont_function <- function(nobs, p_g, M_g, T_exp) {
+      rb <- stats::rbinom(nobs, 1, p_g)
+      grid <- 1:P
+      grid_new <- (grid - min(grid)) / (max(grid) - min(grid))
+      a <- -M_g / (1 - T_exp)
+      y <- a * (grid_new - T_exp)
+      y[y > 0] <- 0
+
+      grid <- 1:P
+      grid_new <- (grid - min(grid)) / (max(grid) - min(grid))
+      a <- -M_g * 0.5 / (1 - T_exp)
+      y2 <- a * (grid_new - T_exp)
+      y[y2 > 0] <- y2[y2 > 0]
+      if (nobs == 1) {
+        out_values <- (y * matrix(rb, P, nobs, byrow = T))
+      } else {
+        out_values <- (y * matrix(rb, P, nobs, byrow = T))
+      }
+
+      return(out_values)
+    }
+  }
+  else if (outlier == "outlier_P") {
+    cont_function <- function(nobs, p_g, M_g, T_exp) {
+      rb <- stats::rbinom(nobs, 1, p_g)
+      (-fun_m(x_seq, 0) + fun_phase(x_seq, M_g)) * matrix(rb, P, nobs, byrow = T)
+    }
+  }
+  if (OC == "no") {
+    out_function <- function(nobs, M_g, T_exp)
+      rep(0, P)
+  }
+  else if (OC == "OC_M") {
+    out_function <- function(nobs, M_g, T_exp) {
+      rep(M_g, P)
+    }
+  }
+  else if (OC == "OC_E") {
+    out_function <- function(nobs, M_g, T_exp) {
+      out_values <- rep(0 , P)
+      grid <- 1:P
+      grid_new <- (grid - min(grid)) / (max(grid) - min(grid))
+      a <- -M_g / (1 - T_exp)
+      y <- a * (grid_new - T_exp)
+      y[y > 0] <- 0
+
+
+      out_values <- rep(0 , P)
+      grid <- 1:P
+      grid_new <- (grid - min(grid)) / (max(grid) - min(grid))
+      a <- -M_g * 0.5 / (1 - T_exp)
+      y2 <- a * (grid_new - T_exp)
+      y[y2 > 0] <- y2[y2 > 0]
+
+      out_values <- (y)
+
+      return(out_values)
+    }
+  }
+  else if (OC == "OC_P") {
+    out_function <- function(nobs, M_g, T_exp) {
+      fun_phase(x_seq, M_g) - f1_m(x_seq, M_g)
+    }
+  }
+  cont_cell_mat <- matrix(0, nobs, P * p)
+  for (ii in 1:p) {
+    index <- 1:P + (ii - 1) * P
+    cont_cell_mat[, index] <-
+      t(cont_function(nobs, p_cellwise, M_outlier_cell, T_exp))
+  }
+  if (max_n_cellwise < p) {
+    cont_cell_mat_which <-
+      sapply(1:10, function(jj)
+        round(rowSums(abs(cont_cell_mat[, P * (jj - 1) + (1:P)])) , 5) > 0)
+    rows_greater_than_allowed <-
+      rowSums(cont_cell_mat_which) > max_n_cellwise
+
+    for (ii in 1:nobs) {
+      if (rows_greater_than_allowed[ii]) {
+        idx <- which(cont_cell_mat_which[ii,])
+        which_to_retain <- sample(idx, max_n_cellwise)
+        which_to_exclude <- setdiff(idx, which_to_retain)
+        for (jj in which_to_exclude) {
+          cont_cell_mat[ii, P * (jj - 1) + (1:P)] <- 0
+        }
+      }
+    }
+  }
+
+  cont_case_mat <- matrix(0, nobs, P * p)
+  ind_out_case <- stats::rbinom(nobs, 1, p_casewise)
+  for (ii in 1:nobs) {
+    if (ind_out_case[ii] == 1) {
+      cont_case_mat[ii,] <-
+        as.numeric(sapply(1:p, function(iii)
+          t(
+            cont_function(1, 1, M_outlier_case, T_exp)
+          )))
+    }
+  }
+  cont_mat <- cont_cell_mat + cont_case_mat
+  ind_out <-
+    lapply(1:p, function(ii)
+      which((abs(
+        rowSums(cont_mat[, 1:P + (ii - 1) * P])
+      ) > 0)))
+  cont_list <- list()
+  for (ii in 1:p) {
+    index <- 1:P + (ii - 1) * P
+    cont_list[[ii]] <- cont_mat[, index]
+  }
+  oc_case_mat <- matrix(0, nobs, P * p)
+  for (ii in 1:nobs) {
+    oc_case_mat[ii, ] <- as.numeric(sapply(1:p, function(jj) {
+      if (!(jj %in% which_OC)) {
+        ret <- rep(0, P)
+      } else {
+        ret <- t(out_function(1, M_OC, T_exp))
+      }
+      return(ret)
+    }))
+
+  }
+  oc_mat <- oc_case_mat
+  oc_list <- list()
+  for (ii in 1:p) {
+    index <- 1:P + (ii - 1) * P
+    oc_list[[ii]] <- oc_mat[, index]
+  }
+
+  X_mat_list <- list()
+  for (ii in 1:p) {
+    X1_scaled <- csi_X %*% t(meig[[ii]])
+    X_mat_list[[ii]] <-
+      t(f1_m(x_seq, M_OC) + t(X1_scaled) * sd) + stats::rnorm(prod(dim(X1_scaled)), sd = sd_e) + cont_list[[ii]] + oc_list[[ii]]
+  }
+  names(X_mat_list) <- paste0("X", sprintf(1:p, fmt = "%02d"))
+  out <- list(X_mat_list = X_mat_list, ind_out = ind_out)
+  return(out)
+}
+
